#  SecretPlots
#  Copyright (c) 2019.  SecretBiology
#
#  Author: Rohit Suratekar
#  Organisation: SecretBiology
#  Website: https://github.com/secretBiology/SecretPlots
#  Licence: MIT License
#  Creation: 11/15/19, 5:10 PM
#
#
# Constant related to the network

# Directions

RIGHT = 0
BOTTOM_RIGHT = 1
BOTTOM = 2
BOTTOM_LEFT = 3
LEFT = 4
TOP_LEFT = 5
TOP = 6
TOP_RIGHT = 7

DIRECTIONS = [
    RIGHT,
    BOTTOM_RIGHT,
    BOTTOM,
    BOTTOM_LEFT,
    LEFT,
    TOP_LEFT,
    TOP,
    TOP_RIGHT
]

DIRECTION_TEXT = [
    "RIGHT",
    "BOTTOM_RIGHT",
    "BOTTOM",
    "BOTTOM_LEFT",
    "LEFT",
    "TOP_LEFT",
    "TOP",
    "TOP_RIGHT"
]
