#  SecretPlots
#  Copyright (c) 2019.  SecretBiology
#
#  Author: Rohit Suratekar
#  Organisation: SecretBiology
#  Website: https://github.com/secretBiology/SecretPlots
#  Licence: MIT License
#  Creation: 03/10/19, 3:20 PM
#
#
# Graph constants

PLOT_BAR = "bar"
PLOT_HIST = "hist"
PLOT_COLOR_MAP = "colormap"
PLOT_STACKED_BAR = "stacked_bar"
PLOT_GROUPED_BAR = "grouped_bar"
PLOT_BOOLEAN_PLOT = "boolean"
PLOT_NETWORK = "network"
